/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.assignJan25;

import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Branch;
import com.sunlabs.archsim.gasp.BranchAction;

/**
 * Branch - two outputs, directs an output to them according to the provided mask.
 * The mask specifies what bit of the address the branch needs to look at to steer the data.
 * @author ib27688
 *
 */
public class MaskBranch extends Branch {
	
	public MaskBranch(String name, String param) {
		super(name, 2, new MaskBranchAction(2, param));
	}

} // Mask Branch

class MaskBranchAction extends BranchAction {
	
	static final String name= "MaskBranchAction";
	
	final int mask;
	
	public MaskBranchAction(int numIn, String param) {
		super(name, numIn);
		int tmpMask= 0;
		try {
			tmpMask= Integer.parseInt(param);
		}
		catch (Exception e) {
			// not an integer
			director.fatalError("MaskBranch " 
					+ "Non integer mask provided: " + param);
			System.exit(1);
		}
		if (tmpMask >= Integer.SIZE) {
			director.fatalError("MaskBranch " 
					+ "The mask provided is larger than the number of bits in an integer: " + param);
			System.exit(1);			
		}
		else if (tmpMask <0) {
			director.fatalError("MaskBranch " 
					+ "The mask provided is negative: " + param);
			System.exit(1);					
		}
		mask= (1 << tmpMask);
	}
	
	public int selectBranch(Object data) {
		int selection= 0;
		if (data instanceof Payload) {
			Payload d= (Payload)data;
				selection = ((d.destination & mask) == 0)? 0 : 1;
		}
		else {
			director.fatalError( name+ " " + getAlias() + ", id= " + getID()
				+ ": Unrecognized object type"
				+ data.getClass().getName() + "instead of Payload");
		}
		return selection;
	}
	
} // MaskBranchAction