/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on November 9, 2005, 3:37 PM
 *
 */

package com.sunlabs.archsim.fleet.ships.tokenadder;

import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;

/**
 *
 * @author mh193489
 */
public class AdderAction extends Action {
        private Delay adderDelay= AdderDefaults.ADDER_DELAY_DEF;
                
        InputTerminal in1;
        InputTerminal in2;
        
    	public AdderAction(String name, CompositeEntity parent) {
		super(name, parent);
		
                // create output functions
                Function outFunction= new AddData();
                Function ackFunction= new AckFunction();

                // bind output functions to terminals
                outputFunctions.put(AdderDefaults.OUT, outFunction);
 		outputFunctions.put(AdderDefaults.ACK_IN1, ackFunction);
 		outputFunctions.put(AdderDefaults.ACK_IN2, ackFunction);
                outputFunctions.put(AdderDefaults.ACK_TOKEN_OUT, ackFunction);

	} // AdderAction
	
        public boolean selfCheck() {
		boolean OK= true;
		boolean check;

                check= (this.getTerminal(AdderDefaults.IN1) != null);
		OK= OK && check;
		if (!check) {
			director.error("Action " + getAlias() + ", id= " + getID()
					+ ", does not have attached terminal in1");
		} else {
                    // store reference to current input for future adding
                    in1= (InputTerminal)this.getTerminal(AdderDefaults.IN1);
                    // bind the appropriate token function for this input
                    outputFunctions.put(AdderDefaults.TOKEN_IN1, new TokenFunction(in1));
                    
                }

                check= (this.getTerminal(AdderDefaults.IN2) != null);
		OK= OK && check;
		if (!check) {
			director.error("Action " + getAlias() + ", id= " + getID()
					+ ", does not have attached terminal in2");
		} else {
                    // store reference to current input for future adding
                    in2= (InputTerminal)this.getTerminal(AdderDefaults.IN2);
                    // bind the appropriate token function for this input
                    outputFunctions.put(AdderDefaults.TOKEN_IN2, new TokenFunction(in2));
                }

                check= (this.getTerminal(AdderDefaults.ACK_OUT) != null);
		OK= OK && check;
		if (!check) {
			director.error("Action " + getAlias() + ", id= " + getID()
					+ ", does not have attached terminal ackOut");
		}
                
		// also run the self check on the superclass
		check= super.selfCheck();
		OK= OK && check;
		
		return OK;
        }
        
        private class AddData extends Function {
		
		public AddData() {
			super(AdderDefaults.ADD_DATA);
			functionDelay= adderDelay;
		}
		
		public Object value() {
                        // check if data on inputs is valid
                        if (in1.getData() == null) {
                                director.journal("AdderAction received payload == null on termianl In1");
                                return null;
                                
                        } else if (in2.getData() == null) {
                                director.journal("AdderAction received payload == null on termianl In2");
                                return null;
                                
                        } else {
                                // get data from input terminals
                                Data data1= ((Payload)in1.getData()).data;                                
                                Data data2= ((Payload)in2.getData()).data;
                                                
                                // check if data inside payload is not null
                                if (data1 == null) {
                                        director.error("AdderAction received empty payload.data == null on terminal In1");
                                        return null;

                                } else if (data2 == null) {
                                        director.error("AdderAction received empty payload.data == null on terminal In2");                                
                                        return null;

                                } else {
                                        // check if input data is valid
                                        if(data1.valid && data2.valid) {
                                                // finally check if data values are not null
                                                if (data1.value == null) {
                                                        director.error("AdderAction received empty payload.data.value == null on terminal In1");
                                                        return null;

                                                } else if (data2.value == null) {
                                                        director.error("AdderAction received empty payload.data.value == null on terminal In2");                                
                                                        return null;

                                                } else {
                                                    
                                                        // both inputs valid, hence compute the sum
                                                        Integer sum= (Integer)data1.value + (Integer)data2.value;
                                                        //check for over or underflow
                                                        if (sum <= AdderDefaults.UNDERFLOW) {
                                                            return new Data(AdderDefaults.MINUS_INFINITY,false);

                                                        } else if (AdderDefaults.OVERFLOW <= sum) {
                                                            return new Data(AdderDefaults.PLUS_INFINITY, false);

                                                        } else {
                                                            return new Data(sum, true);
                                                        }
                                                }
                                                
                                        } else {
                                            // at least one input invalid, hence return invalid sum
                                            return new Data(AdderDefaults.OOB, false);
                                        }
                                        
                                }
                        }
		} // value
	} // AddData

}


